;; (find-efile "etags.el")
'(load-library "etags")
(require 'etags)			; we need to set tags-add-tables to nil
(setq tags-add-tables nil)		; for find-___tags

;; TO DO: check if we should use (let (tags-add-tables) ...) instead.

(defun format-and-eval (formatstr &rest rest)
  "Apply `format' to FORMATSTR and REST and evaluate the result."
  (eval (read (apply 'format (concat "(progn " formatstr ")") rest))))

(defun find-node2 (infofile nodename &optional pos-spec-list  code)
  "Like `find-node', but saves CODE in `ee-temp-code' (to let `inn' use it)."
  (if code (setq ee-temp-code code ee-temp-infofile infofile))
  (apply 'find-node (format "(%s)%s" infofile nodename) pos-spec-list))

;;;###autoload
(defun code-c-d (c d &optional infofile)
  "Define an entire family of hyperlink functions at once.
A call to `code-c-d' with \"xxx\" as the value of C will define the
hyperlink functions `find-xxxfile', `find-xxxtag', `find-xxxw3' and
optionally `find-xxxnode', plus the auxiliary functions and variables
`ee-xxxdir', `ee-xxxtagsfile', `ee-xxxfile' and `ee-use-xxx-tags'.\n
To best way to understand this function is to make a call to
`icode-c-d' with the same parameters to see the code that `code-c-d'
will execute.  To see an example, copy the two lines below to a
non-read-only buffer and execute each of them with \\[eval-last-sexp]:\n
  # (icode-c-d \"grub\" \"/usr/src/grub-0.5.95/\")
  # (icode-c-d \"grub\" \"/usr/src/grub-0.5.95/\" \"grub\")\n
For more information (examples, tutorials, etc) see
<http://angg.twu.net/>."
  (format-and-eval "
    (setq ee-%sdir \"%s\")
    (setq ee-%stagsfile \"%sTAGS\")
    (defun ee-%sfile (str)
      (concat (substitute-in-file-name ee-%sdir) str))
    (defun ee-use-%s-tags ()
      (setq tags-file-name ee-%stagsfile))
    (defun find-%sfile (str &rest pos-spec-list)
      (ee-use-%s-tags)
      (apply 'find-fline-nosubst (ee-%sfile str) pos-spec-list))
    (defun find-%stag (str)
      (ee-use-%s-tags) (find-tag str))
    (defun find-%sw3 (furl &rest pos-spec-list)
      (apply 'find-w3 (ee-%sfile furl) pos-spec-list))
    (setq ee-temp-code %S ee-temp-infofile %S)
    " c d  c d  c c  c c  c c c  c c  c c  c infofile)
  (if infofile
     (format-and-eval "
       (defun find-%snode (nodename &rest pos-spec-list)
          (find-node2 %S nodename pos-spec-list  %S))" c infofile  c)))





(defun code-ps (code psfile)
  (format-and-eval
   "(defun find-%spage (n &rest comments) (find-pspage %S n))"
   code psfile))
(defun code-dvi (code dvifile)
  (format-and-eval
   "(defun find-%spage (n &rest comments) (find-dvipage %S n))"
   code dvifile))




;;;;
;;;; ee-insert-info-hyperlink
;;;; Also called:         inn  (to ".inn")
;;;; Insert a hyperlink to the current info node.
;;;; 
;;
;; (find-elnode "Interactive Examples")
;; (find-efile "info.el")
;; (find-efile "info.el" "Convert filename to lower case")
;; (find-etag "Info-set-mode-line")
;;
(defun ee-infofile-to-code (infofile)
  "Try to convert INFOFILE to a \"code\" that can be used by `inn'.
This function is quite primitive at present: it just takes the
non-directory parts of INFOFILE and of `ee-temp-infofile', and if they
are equal it returns `ee-temp-code' (else it returns nil)."
  (if (and infofile
	   ee-temp-infofile
	   (string= (file-name-nondirectory infofile)
		    (file-name-nondirectory ee-temp-infofile)))
      ee-temp-code))

(defun ee-insert-info-hyperlink (arg)
  "Insert a hyperlink to the current info node.
If ARG is non-nil try to use the short form, like `# (find-elnode \"Top\")'.
Without an argument always use long forms, like `(find-node \"(elisp)Top\")'.
The infofile part (\"(elisp)\" in the example) is obtained by just taking
the non-directory part of `Info-current-file', and thus may be incomplete
if the infofile was at a non-standard directory; use `code-c-d' to work
around that."
  (interactive "P")
  (let (infofile infonode f code s)
    (if (get-buffer "*info*")
	(progn
	  (save-excursion
	    (set-buffer "*info*")
	    (setq infofile Info-current-file
		  infonode Info-current-node
		  f (file-name-nondirectory infofile)))
	  (if arg (setq code (ee-infofile-to-code infofile)))
	  ;; (insert (format "dbg: %S %S %S\n" infofile ee-temp-infofile f))
	  (insert
	   (format ee-comment-format
		   (if code (format "(find-%snode \"%s\")" code infonode)
		     (format "(find-node \"(%s)%s\")" f infonode)))))
      (error "No curent info node"))))


;;;;
;;;; ee-convert-to-debian-link
;;;; Also called:          dff  (to ".dff")
;;;; This is a macro to create find-xxx hyperlinks for debian packages.
;;;;
;;
;; (find-elnode "Keyboard Macros")
;;
(defun ee-convert-to-debian-link (N)
  "Convert N names of Debian packages into hyperlinks to the packages.
For example, if you have two lines like:\n
  grub
  oskit\n
in a non-read-only buffer and the point is on the first one, then
running `M-2 M-x dff' will convert them to:\n
  # (find-status \"grub\")
  # (find-vldifile \"grub.list\")
  # (find-fline \"/usr/doc/grub/\")
  # (find-status \"oskit\")
  # (find-vldifile \"oskit.list\")
  # (find-fline \"/usr/doc/oskit/\")\n"
  (interactive "p")
  (execute-kbd-macro
   (read-kbd-macro "C-a NUL C-e C-w
                    # SPC (find-status SPC \" C-y \") RET
                    # SPC (find-vldifile SPC \" C-y .list\") RET
                    # SPC (find-fline SPC \"/usr/doc/ C-y /\") C-a <down>")
   N))
